@@ -27,13 +27,10 @@
 
 #include <string.h>
 #include <unistd.h>
-#include <popt.h>
-
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-
 #include <libgnomeui/libgnomeui.h>
 #include <glade/glade.h>
 
@@ -47,12 +44,16 @@
 
 static void gpm_exit (GpmManager *manager);
 
-/** registers org.gnome.PowerManager on a connection
+/**
+ * gpm_object_register:
+ * @connection: What we want to register to
+ * @object: The GObject we want to register
  *
- *  @return			If we successfully registered the object
+ * Register org.gnome.PowerManager on the session bus.
+ * This function MUST be called before DBUS service will work.
  *
- *  @note	This function MUST be called before DBUS service will work.
- */
+ * Return value: success
+ **/
 static gboolean
 gpm_object_register (DBusGConnection *connection,
 		     GObject         *object)
@@ -65,23 +66,7 @@ gpm_object_register (DBusGConnection *connection,
 					       DBUS_SERVICE_DBUS,
 					       DBUS_PATH_DBUS,
 					       DBUS_INTERFACE_DBUS);
-/*
- * Add this define hack until we depend on DBUS 0.60, as the
- * define names have changed.
- * should fix bug: http://bugzilla.gnome.org/show_bug.cgi?id=322435
- */
-#ifdef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
-	/* add the legacy stuff for FC4 */
-	if (!dbus_g_proxy_call (bus_proxy, "RequestName", &error,
-		G_TYPE_STRING, GPM_DBUS_SERVICE,
-		G_TYPE_UINT, DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT,
-		G_TYPE_INVALID,
-		G_TYPE_UINT, &request_name_result,
-		G_TYPE_INVALID)) {
-		g_warning ("Failed to acquire %s: %s", GPM_DBUS_SERVICE, error->message);
-		return FALSE;
-	}
-#else
+
 	if (!dbus_g_proxy_call (bus_proxy, "RequestName", &error,
 		G_TYPE_STRING, GPM_DBUS_SERVICE,
 		G_TYPE_UINT, 0,
@@ -91,7 +76,6 @@ gpm_object_register (DBusGConnection *connection,
 		g_warning ("Failed to acquire %s: %s", GPM_DBUS_SERVICE, error->message);
 		return FALSE;
 	}
-#endif
 
  	if (request_name_result != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER)
 		return FALSE;
@@ -105,6 +89,10 @@ gpm_object_register (DBusGConnection *connection,
 	return TRUE;
 }
 
+/**
+ * gpm_exit:
+ * @manager: This manager class instance
+ **/
 static void
 gpm_exit (GpmManager *manager)
 {
@@ -115,68 +103,47 @@ gpm_exit (GpmManager *manager)
 	exit (0);
 }
 
-static void
-gpm_critical_error (const char *content)
-{
-	GtkWidget *dialog;
-	dialog = gtk_message_dialog_new_with_markup (NULL,
-						     GTK_DIALOG_MODAL,
-						     GTK_MESSAGE_WARNING,
-						     GTK_BUTTONS_CLOSE,
-						     "<span size='larger'><b>%s</b></span>",
-						     GPM_NAME);
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog),
-						    content);
-	/* we close the gtk lopp when the user clicks close */
-	g_signal_connect_swapped (dialog,
-				  "response",
-				  G_CALLBACK (gtk_main_quit),
-				  NULL);
-	gtk_window_present (GTK_WINDOW (dialog));
-	/* we wait here for user to click close */
-	gtk_main();
-}
-
-/** Main entry point
- *
- *  @param	argc		Number of arguments given to program
- *  @param	argv		Arguments given to program
- *  @return			Return code
- */
+/**
+ * main:
+ **/
 int
 main (int argc, char *argv[])
 {
-	gint             i;
 	GMainLoop       *loop;
 	GnomeClient     *master;
 	GnomeClientFlags flags;
 	DBusGConnection *system_connection;
 	DBusGConnection *session_connection;
 	gboolean         verbose = FALSE;
-	gboolean         no_daemon;
+	gboolean         no_daemon = FALSE;
 	GpmManager      *manager = NULL;
 	GError          *error = NULL;
+	GOptionContext  *context;
+ 	GnomeProgram    *program;
 
-	struct poptOption options[] = {
-		{ "no-daemon", '\0', POPT_ARG_NONE, NULL, 0,
+
+	const GOptionEntry options[] = {
+		{ "no-daemon", '\0', 0, G_OPTION_ARG_NONE, &no_daemon,
 		  N_("Do not daemonize"), NULL },
-		{ "verbose", '\0', POPT_ARG_NONE, NULL, 0,
+		{ "verbose", '\0', 0, G_OPTION_ARG_NONE, &verbose,
 		  N_("Show extra debugging information"), NULL },
-		{ NULL, '\0', 0, NULL, 0, NULL, NULL }
+		{ NULL}
 	};
 
-	i = 0;
-	options[i++].arg = &no_daemon;
-	options[i++].arg = &verbose;
+	context = g_option_context_new (_("GNOME Power Manager"));
 
-	no_daemon = FALSE;
+	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (argv[0], VERSION,
-			    LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PROGRAM_STANDARD_PROPERTIES,
-			    GNOME_PARAM_POPT_TABLE, options,
-			    GNOME_PARAM_HUMAN_READABLE_NAME, GPM_NAME,
-			    NULL);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
+	program = gnome_program_init (argv[0], VERSION,
+			   	      LIBGNOMEUI_MODULE, argc, argv,
+			    	      GNOME_PROGRAM_STANDARD_PROPERTIES,
+			    	      GNOME_PARAM_GOPTION_CONTEXT, context,
+			    	      GNOME_PARAM_HUMAN_READABLE_NAME, GPM_NAME,
+			    	      NULL);
 
 	master = gnome_master_client ();
 	flags = gnome_client_get_flags (master);
@@ -193,9 +160,6 @@ main (int argc, char *argv[])
 		g_thread_init (NULL);
 	dbus_g_thread_init ();
 
-	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
 
 	gpm_debug_init (verbose);
 
@@ -230,11 +194,11 @@ main (int argc, char *argv[])
 	}
 
 	if (! gpm_stock_icons_init()) {
-		g_error ("Cannot continue without stock icons");
+		gpm_critical_error ("Cannot continue without stock icons");
 	}
 
 	if (! no_daemon && daemon (0, 0)) {
-		g_error ("Could not daemonize: %s", g_strerror (errno));
+		gpm_critical_error ("Could not daemonize: %s", g_strerror (errno));
 	}
 
 	manager = gpm_manager_new ();
@@ -252,5 +216,8 @@ main (int argc, char *argv[])
 
 	gpm_exit (manager);
 
+	g_object_unref (program);
+	g_option_context_free (context);
+
 	return 0;
 }
