@@ -29,6 +29,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 
 #include "gpm-common.h"
 #include "gpm-hal.h"
@@ -37,6 +38,8 @@
 #include "gpm-power.h"
 #include "gpm-marshal.h"
 #include "gpm-debug.h"
+#include "gpm-prefs.h"
+#include "gpm-dbus-system-monitor.h"
 
 static void     gpm_power_class_init (GpmPowerClass *klass);
 static void     gpm_power_init       (GpmPower      *power);
@@ -46,12 +49,12 @@ static void     gpm_power_finalize   (GObject       *object);
 
 struct GpmPowerPrivate
 {
-	gboolean       on_ac;
-
-	GHashTable    *battery_kind_cache;
-	GHashTable    *battery_device_cache;
-
-	GpmHalMonitor *hal_monitor;
+	gboolean		 on_ac;
+	int			 exp_ave_factor;
+	GHashTable		*battery_kind_cache;
+	GHashTable		*battery_device_cache;
+	GpmDbusSystemMonitor	*dbus_system;
+	GpmHalMonitor		*hal_monitor;
 };
 
 enum {
@@ -74,7 +77,6 @@ enum {
 	PROP_ON_AC
 };
 
-static GObjectClass *parent_class = NULL;
 static guint	     signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmPower, gpm_power, G_TYPE_OBJECT)
@@ -94,21 +96,21 @@ typedef struct {
 
 typedef struct {
 	char		       *udi;
-	int			charge_rate_previous;
-	GpmPowerBatteryKind	battery_kind;
-	GpmPowerBatteryStatus	battery_status;
 	char		       *product;
 	char		       *vendor;
 	char		       *technology;
-	GpmPowerBatteryUnit	unit;
 	char		       *serial;
 	char		       *model;
+	int			charge_rate_previous;
+	GpmPowerBatteryKind	battery_kind;
+	GpmPowerBatteryStatus	battery_status;
+	GpmPowerBatteryUnit	unit;
 } BatteryDeviceCacheEntry;
 
-/* Increasing this value will increase the damping effect
- * of the rate calculations, 0.8 seems a good default. */
-#define		RATE_EXP_AVERAGE_FACTOR			(0.80f)
-
+/**
+ * gpm_power_battery_status_set_defaults:
+ * @status: A battery info structure
+ **/
 static void
 gpm_power_battery_status_set_defaults (GpmPowerBatteryStatus *status)
 {
@@ -116,7 +118,8 @@ gpm_power_battery_status_set_defaults (GpmPowerBatteryStatus *status)
 	status->design_charge = 0;
 	status->last_full_charge = 0;
 	status->current_charge = 0;
-	status->charge_rate = 0;
+	status->charge_rate_smoothed = 0;
+	status->charge_rate_raw = 0;
 	status->percentage_charge = 0;
 	status->remaining_time = 0;
 	status->capacity = 0;
@@ -126,14 +129,19 @@ gpm_power_battery_status_set_defaults (GpmPowerBatteryStatus *status)
 	status->is_discharging = FALSE;
 }
 
-/* we have to be clever here, as there are lots of broken batteries */
+/**
+ * gpm_power_battery_is_charged:
+ * @status: A battery info structure
+ *
+ * We have to be clever here, as there are lots of broken batteries.
+ * Some batteries have a reduced charge capacity and cannot charge to
+ * 100% anymore (although they *should* update thier own last_full
+ * values, it appears most do not...)
+ * Return value: If the battery is considered "charged".
+ **/
 gboolean
 gpm_power_battery_is_charged (GpmPowerBatteryStatus *status)
 {
-	/* We have to do the additional check for 90% as
-	   some batteries have a reduced charge capacity and cannot
-	   charge to 100% anymore (although they *should* update
-	   thier own last_full tags, it appears most do not...) */
 	if (! status->is_charging &&
 	    ! status->is_discharging &&
 	    status->percentage_charge > 90) {
@@ -142,6 +150,12 @@ gpm_power_battery_is_charged (GpmPowerBatteryStatus *status)
 	return FALSE;
 }
 
+/**
+ * battery_device_cache_entry_update_all:
+ * @entry: A device cache instance
+ *
+ * Updates all the information fields in a cache entry
+ **/
 static void
 battery_device_cache_entry_update_all (BatteryDeviceCacheEntry *entry)
 {
@@ -211,21 +225,22 @@ battery_device_cache_entry_update_all (BatteryDeviceCacheEntry *entry)
 	/* sanity check that charge_level.rate exists (if it should) */
 	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY) {
 		exists = gpm_hal_device_get_int (udi, "battery.charge_level.rate",
-						 &status->charge_rate);
+						 &status->charge_rate_raw);
 		if (!exists && (status->is_discharging || status->is_charging)) {
 			gpm_warning ("could not read your battery's charge rate");
 		}
-
+		if (exists) {
+			status->charge_rate_smoothed = status->charge_rate_raw;
+		}
 		/* FIXME: following can be removed if bug #5752 of hal on freedesktop
 		   gets fixed and is part of a new release of HAL and we depend on that
 		   version*/
-		if (exists && status->charge_rate == 0) {
+		if (exists && status->charge_rate_raw == 0) {
 			status->is_discharging = FALSE;
 			status->is_charging = FALSE;
 		}
 	}
 
-
 	/* sanity check that charge_level.percentage exists (if it should) */
 	exists = gpm_hal_device_get_int (udi, "battery.charge_level.percentage",
 					     &status->percentage_charge);
@@ -248,8 +263,8 @@ battery_device_cache_entry_update_all (BatteryDeviceCacheEntry *entry)
 		if (status->design_charge > 0 && status->last_full_charge > 0) {
 			if (status->design_charge != status->last_full_charge) {
 				float capacity;
-				capacity = status->design_charge / status->last_full_charge;
-				status->capacity = capacity * 100;
+				capacity = 100.0f / (float) status->design_charge;
+				status->capacity = capacity * (float) status->last_full_charge;
 			}
 		}
 	}
@@ -273,9 +288,48 @@ battery_device_cache_entry_update_all (BatteryDeviceCacheEntry *entry)
 	}
 }
 
+/**
+ * gpm_power_exp_aver:
+ * @previous: The old value
+ * @new: The new value
+ * @factor_pc: The factor as a percentage
+ *
+ * We should do an exponentially weighted average so that high frequency
+ * changes are smoothed. This should mean the rate (and thus the remaining time)
+ * does not change drastically between updates.
+ **/
+static int
+gpm_power_exp_aver (int previous, int new, int factor_pc)
+{
+	int result = 0;
+	float factor = 0;
+	float factor_inv = 1;
+	if (previous == 0 || factor_pc == 0) {
+		/* startup, or re-initialization - we have no data */
+		gpm_debug ("Telling rate with no ave factor!");
+		result = new;
+	} else {
+		factor = (float) factor_pc / 100.0f;
+		factor_inv = 1.0f - factor;
+		result = (int) ((factor_inv * (float) new) + (factor * (float) previous));
+	}
+	gpm_debug ("factor = %f, previous = %i, new=%i, result = %i",
+		   factor, previous, new, result);
+	return result;
+}
+
+/**
+ * battery_device_cache_entry_update_key:
+ * @power: This power class instance
+ * @entry: A device cache instance
+ * @key: the HAL key name, e.g. battery.rechargeable.is_charging
+ *
+ * Updates the device object with the new information given to us from HAL.
+ **/
 static void
-battery_device_cache_entry_update_key (BatteryDeviceCacheEntry *entry,
-				       const char	      *key)
+battery_device_cache_entry_update_key (GpmPower		       *power,
+				       BatteryDeviceCacheEntry *entry,
+				       const char	       *key)
 {
 	GpmPowerBatteryStatus *status = &entry->battery_status;
 	char *udi = entry->udi;
@@ -287,20 +341,16 @@ battery_device_cache_entry_update_key (BatteryDeviceCacheEntry *entry,
 	/* update values in the struct */
 	if (strcmp (key, "battery.present") == 0) {
 		gpm_hal_device_get_bool (udi, key, &status->is_present);
-
 		battery_device_cache_entry_update_all (entry);
 
 	} else if (strcmp (key, "battery.rechargeable.is_charging") == 0) {
 		gpm_hal_device_get_bool (udi, key, &status->is_charging);
+		status->charge_rate_smoothed = 0;
 
-		 /* invalidate the remaining time, as we need to wait for
-		    the next HAL update. This is a HAL bug I think. */
-		status->remaining_time = 0;
 	} else if (strcmp (key, "battery.rechargeable.is_discharging") == 0) {
 		gpm_hal_device_get_bool (udi, key, &status->is_discharging);
+		status->charge_rate_smoothed = 0;
 
-		/* invalidate the remaining time */
-		status->remaining_time = 0;
 	} else if (strcmp (key, "battery.charge_level.design") == 0) {
 		gpm_hal_device_get_int (udi, key, &status->design_charge);
 
@@ -311,25 +361,18 @@ battery_device_cache_entry_update_key (BatteryDeviceCacheEntry *entry,
 		gpm_hal_device_get_int (udi, key, &status->current_charge);
 
 	} else if (strcmp (key, "battery.charge_level.rate") == 0) {
-		int charge_rate_new;
-		gpm_hal_device_get_int (udi, key, &charge_rate_new);
-		/* Do an exponentially weighted average for the rate so
-		   that high frequency changes are smoothed. This should mean
-		   the remaining_time does not change drastically between updates.
-		   Fixes bug #328927 */
-		if (entry->charge_rate_previous == 0) {
-			/* startup, or re-initialization - we have no data */
-			status->charge_rate = charge_rate_new;
-		} else {
-			status->charge_rate = ((1.0f - RATE_EXP_AVERAGE_FACTOR) * charge_rate_new) +
-					       (RATE_EXP_AVERAGE_FACTOR * entry->charge_rate_previous);
-		}
-		entry->charge_rate_previous = charge_rate_new;
+		gpm_hal_device_get_int (udi, key, &status->charge_rate_raw);
+
+		/* Do an exponentially weighted average, fixes bug #328927 */
+		status->charge_rate_smoothed = gpm_power_exp_aver (entry->charge_rate_previous,
+							status->charge_rate_raw,
+							power->priv->exp_ave_factor);
+		entry->charge_rate_previous = status->charge_rate_smoothed;
 
 		/* FIXME: following can be removed if bug #5752 of hal on freedesktop
 		   gets fixed and is part of a new release of HAL and we depend on that
 		   version */
-		if (status->charge_rate == 0) {
+		if (status->charge_rate_raw == 0) {
 			status->is_discharging = FALSE;
 			status->is_charging = FALSE;
 		}
@@ -345,6 +388,13 @@ battery_device_cache_entry_update_key (BatteryDeviceCacheEntry *entry,
 	}
 }
 
+/**
+ * battery_device_cache_entry_new_from_udi:
+ * @udi: The HAL UDI for this device
+ *
+ * Creates a new device object and populates the values.
+ * Return value: The new device cache entry.
+ **/
 static BatteryDeviceCacheEntry *
 battery_device_cache_entry_new_from_udi (const char *udi)
 {
@@ -366,18 +416,24 @@ battery_device_cache_entry_new_from_udi (const char *udi)
 	return entry;
 }
 
+/**
+ * battery_kind_cache_entry_new_from_battery_kind:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ **/
 static BatteryKindCacheEntry *
 battery_kind_cache_entry_new_from_battery_kind (GpmPowerBatteryKind battery_kind)
 {
 	BatteryKindCacheEntry *entry;
-
 	entry = g_new0 (BatteryKindCacheEntry, 1);
-
 	entry->battery_kind = battery_kind;
-
 	return entry;
 }
 
+/**
+ * battery_kind_cache_entry_free:
+ * @entry: A device cache instance
+ **/
 static void
 battery_kind_cache_entry_free (BatteryKindCacheEntry *entry)
 {
@@ -387,6 +443,11 @@ battery_kind_cache_entry_free (BatteryKindCacheEntry *entry)
 	entry = NULL;
 }
 
+/**
+ * battery_kind_to_string:
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ * Return value: The localised battery kind. Do not free this string.
+ **/
 const char *
 battery_kind_to_string (GpmPowerBatteryKind battery_kind)
 {
@@ -408,6 +469,10 @@ battery_kind_to_string (GpmPowerBatteryKind battery_kind)
 	return str;
 }
 
+/**
+ * battery_kind_cache_debug_print:
+ * @entry: A device cache instance
+ **/
 static void
 battery_kind_cache_debug_print (BatteryKindCacheEntry *entry)
 
@@ -420,14 +485,17 @@ battery_kind_cache_debug_print (BatteryKindCacheEntry *entry)
 		   status->is_present, status->last_full_charge);
 	gpm_debug ("percent    %i\tcurrent    %i",
 		   status->percentage_charge, status->current_charge);
-	gpm_debug ("charge     %i\trate       %i", 
-		   status->is_charging, status->charge_rate);
-	gpm_debug ("discharge  %i\tremaining  %i", 
+	gpm_debug ("charge     %i\trate (raw) %i",
+		   status->is_charging, status->charge_rate_raw);
+	gpm_debug ("discharge  %i\tremaining  %i",
 		   status->is_discharging, status->remaining_time);
-	gpm_debug ("capacity   %i", 
+	gpm_debug ("capacity   %i",
 		   status->capacity);
 }
 
+/**
+ * debug_print_type_cache_iter:
+ **/
 static void
 debug_print_type_cache_iter (gpointer	       key,
 			     BatteryKindCacheEntry *entry,
@@ -436,6 +504,10 @@ debug_print_type_cache_iter (gpointer	       key,
 	battery_kind_cache_debug_print (entry);
 }
 
+/**
+ * battery_kind_cache_debug_print_all:
+ * @power: This power class instance
+ **/
 static void
 battery_kind_cache_debug_print_all (GpmPower *power)
 {
@@ -446,6 +518,15 @@ battery_kind_cache_debug_print_all (GpmPower *power)
 	}
 }
 
+/**
+ * battery_device_cache_find:
+ * @power: This power class instance
+ * @udi: The HAL UDI for this device
+ *
+ * Finds the UDI in the device cache.
+ *
+ * Return value: The entry if found, or NULL if missing.
+ **/
 static BatteryDeviceCacheEntry *
 battery_device_cache_find (GpmPower   *power,
 			   const char *udi)
@@ -466,6 +547,15 @@ battery_device_cache_find (GpmPower   *power,
 	return entry;
 }
 
+/**
+ * battery_kind_cache_find:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ *
+ * Finds the battery kind in the kind cache.
+ *
+ * Return value: The entry if found, or NULL if missing.
+ **/
 static BatteryKindCacheEntry *
 battery_kind_cache_find (GpmPower		*power,
 			 GpmPowerBatteryKind	 battery_kind)
@@ -481,7 +571,13 @@ battery_kind_cache_find (GpmPower		*power,
 	return entry;
 }
 
-/** returns the number of devices of a specific kind */
+/**
+ * gpm_power_get_num_devices_of_kind:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ *
+ * Return value: the number of devices of a specific kind.
+ **/
 gint
 gpm_power_get_num_devices_of_kind (GpmPower		*power,
 				   GpmPowerBatteryKind	 battery_kind)
@@ -497,7 +593,12 @@ gpm_power_get_num_devices_of_kind (GpmPower		*power,
 	return (g_slist_length (entry->devices));
 }
 
-/** frees the custom array type */
+/**
+ * gpm_power_free_description_array:
+ * @array: This description array
+ *
+ * Frees the custom array type.
+ **/
 void
 gpm_power_free_description_array (GArray *array)
 {
@@ -511,6 +612,11 @@ gpm_power_free_description_array (GArray *array)
 	g_array_free (array, TRUE);
 }
 
+/**
+ * get_power_unit_suffix:
+ * @unit: The unit type, e.g. GPM_POWER_UNIT_MWH
+ * Return value: The localised device type. Do not free this.
+ **/
 static const char *
 get_power_unit_suffix (GpmPowerBatteryUnit unit)
 {
@@ -527,7 +633,16 @@ get_power_unit_suffix (GpmPowerBatteryUnit unit)
 	return suffix;
 }
 
-/** returns in a custom array the device parameters */
+/**
+ * gpm_power_get_description_array:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ * @device_num: The device number (e.g. 1 for the second primary battery)
+ *
+ * Returns in a custom array the device parameters
+ * You need to free the resulting array with gpm_power_free_description_array().
+ * Return value: A description array.
+ **/
 GArray *
 gpm_power_get_description_array (GpmPower		*power,
 				 GpmPowerBatteryKind	 battery_kind,
@@ -631,11 +746,13 @@ gpm_power_get_description_array (GpmPower		*power,
 		const char *condition;
 		di.title = g_strdup (_("Capacity:"));
 		if (status->capacity > 99) {
-			condition = _("Very good condition");
+			condition = _("Excellent");
 		} else if (status->capacity > 90) {
-			condition = _("Good condition");
+			condition = _("Good");
+		} else if (status->capacity > 70) {
+			condition = _("Fair");
 		} else {
-			condition = _("Poor condition");
+			condition = _("Poor");
 		}
 		di.value = g_strdup_printf ("%i%% (%s)", status->capacity, condition);
 		g_array_append_vals (array, &di, 1);
@@ -659,9 +776,14 @@ gpm_power_get_description_array (GpmPower		*power,
 			di.value = g_strdup_printf ("%i%s", status->design_charge, suffix);
 			g_array_append_vals (array, &di, 1);
 		}
-		if (status->charge_rate > 0) {
-			di.title = g_strdup (_("Charge rate:"));
-			di.value = g_strdup_printf ("%imWh", status->charge_rate);
+		if (status->charge_rate_raw > 0) {
+			di.title = g_strdup (_("Charge rate (raw):"));
+			di.value = g_strdup_printf ("%imWh", status->charge_rate_raw);
+			g_array_append_vals (array, &di, 1);
+		}
+		if (status->charge_rate_smoothed > 0) {
+			di.title = g_strdup (_("Charge rate (smoothed):"));
+			di.value = g_strdup_printf ("%imWh", status->charge_rate_smoothed);
 			g_array_append_vals (array, &di, 1);
 		}
 	}
@@ -669,6 +791,17 @@ gpm_power_get_description_array (GpmPower		*power,
 	return array;
 }
 
+/**
+ * battery_kind_cache_update:
+ * @power: This power class instance
+ * @entry: A kind cache instance
+ *
+ * Updates the kind cache from the constituent device cache objects. This is
+ * needed on multibattery laptops where the time needs to be computed over
+ * two or more battereies. Some laptop batteries discharge one after the other,
+ * some discharge simultanously.
+ * This also does sanity checking on the values to make sure they are sane.
+ **/
 static void
 battery_kind_cache_update (GpmPower	         *power,
 			   BatteryKindCacheEntry *entry)
@@ -714,7 +847,8 @@ battery_kind_cache_update (GpmPower	         *power,
 		type_status->design_charge += device_status->design_charge;
 		type_status->last_full_charge += device_status->last_full_charge;
 		type_status->current_charge += device_status->current_charge;
-		type_status->charge_rate += device_status->charge_rate;
+		type_status->charge_rate_smoothed += device_status->charge_rate_smoothed;
+		type_status->charge_rate_raw += device_status->charge_rate_raw;
 		/* we have to sum this here, in case the device has no rate
 		   data, and we can't compute it further down */
 		type_status->remaining_time += device_status->remaining_time;
@@ -747,14 +881,14 @@ battery_kind_cache_update (GpmPower	         *power,
 	}
 	/* We only do the "better" remaining time algorithm if the battery has rate,
 	   i.e not a UPS, which gives it's own battery.remaining_time but has no rate */
-	if (type_status->charge_rate > 0) {
+	if (type_status->charge_rate_smoothed > 0) {
 		if (type_status->is_discharging) {
 			type_status->remaining_time = 3600 * ((float)type_status->current_charge /
-							      (float)type_status->charge_rate);
+							      (float)type_status->charge_rate_smoothed);
 		} else if (type_status->is_charging) {
 			type_status->remaining_time = 3600 *
 				((float)(type_status->last_full_charge - type_status->current_charge) /
-				(float)type_status->charge_rate);
+				(float)type_status->charge_rate_smoothed);
 		}
 	}
 	/* Check the remaining time is under a set limit, to deal with broken
@@ -770,6 +904,9 @@ battery_kind_cache_update (GpmPower	         *power,
 	g_signal_emit (power, signals [BATTERY_STATUS_CHANGED], 0, entry->battery_kind);
 }
 
+/**
+ * battery_kind_update_cache_iter:
+ **/
 static void
 battery_kind_update_cache_iter (const char	      *key,
 				BatteryKindCacheEntry *entry,
@@ -778,6 +915,12 @@ battery_kind_update_cache_iter (const char	      *key,
 	battery_kind_cache_update (power, entry);
 }
 
+/**
+ * battery_kind_cache_update_all:
+ * @power: This power class instance
+ *
+ * Updates every device of every type
+ **/
 static void
 battery_kind_cache_update_all (GpmPower *power)
 {
@@ -790,6 +933,11 @@ battery_kind_cache_update_all (GpmPower *power)
 	}
 }
 
+/**
+ * battery_device_cache_add_device:
+ * @power: This power class instance
+ * @entry: A device cache instance
+ **/
 static void
 battery_device_cache_add_device (GpmPower		 *power,
 				 BatteryDeviceCacheEntry *entry)
@@ -799,6 +947,11 @@ battery_device_cache_add_device (GpmPower		 *power,
 			     entry);
 }
 
+/**
+ * battery_device_cache_remove_device:
+ * @power: This power class instance
+ * @entry: A device cache instance
+ **/
 static void
 battery_device_cache_remove_device (GpmPower		    *power,
 				    BatteryDeviceCacheEntry *entry)
@@ -813,6 +966,13 @@ battery_device_cache_remove_device (GpmPower		    *power,
 	g_free (entry->model);
 }
 
+/**
+ * battery_kind_cache_add_device:
+ * @power: This power class instance
+ * @device_entry: A device cache instance
+ *
+ * Adds a device entry to the correct cache entry.
+ **/
 static void
 battery_kind_cache_add_device (GpmPower			*power,
 			       BatteryDeviceCacheEntry	*device_entry)
@@ -838,6 +998,13 @@ battery_kind_cache_add_device (GpmPower			*power,
 	battery_kind_cache_update (power, type_entry);
 }
 
+/**
+ * battery_kind_cache_remove_device:
+ * @power: This power class instance
+ * @entry: A device cache instance
+ *
+ * Removes a device entry from a cache entry.
+ **/
 static void
 battery_kind_cache_remove_device (GpmPower		  *power,
 				  BatteryDeviceCacheEntry *entry)
@@ -861,6 +1028,15 @@ battery_kind_cache_remove_device (GpmPower		  *power,
 	}
 }
 
+/**
+ * power_get_summary_for_battery_kind:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ * @summary: The returned string summary
+ *
+ * Gets a summary for a specific battery kind. This is used on the tooltip
+ * mainly.
+ **/
 static void
 power_get_summary_for_battery_kind (GpmPower		*power,
 			    	    GpmPowerBatteryKind  battery_kind,
@@ -930,13 +1106,20 @@ power_get_summary_for_battery_kind (GpmPower		*power,
 		gpm_warning ("in an undefined state we are not charging or "
 			     "discharging and the batteries are also not charged");
 	}
-	
+
 	/* append percentage to all devices */
 	g_string_append_printf (summary, " (%i%%)\n", status->percentage_charge);
 
 	g_free (timestring);
 }
 
+/**
+ * gpm_power_get_status_summary:
+ * @power: This power class instance
+ * @string: The returned string
+ *
+ * Returns the complete tooltip ready for display. Text logic is done here :-).
+ **/
 gboolean
 gpm_power_get_status_summary (GpmPower *power,
 			      char    **string,
@@ -983,10 +1166,17 @@ gpm_power_get_status_summary (GpmPower *power,
 	return TRUE;
 }
 
-/* returns if the device was found in the cache */
+/**
+ * gpm_power_get_battery_status:
+ * @power: This power class instance
+ * @battery_kind: The type of battery, e.g. GPM_POWER_BATTERY_KIND_PRIMARY
+ * @battery_status: A battery info structure
+ *
+ * Return value: if the device was found in the cache.
+ **/
 gboolean
 gpm_power_get_battery_status (GpmPower			 *power,
-			      GpmPowerBatteryKind	 battery_kind,
+			      GpmPowerBatteryKind	  battery_kind,
 			      GpmPowerBatteryStatus      *battery_status)
 {
 	BatteryKindCacheEntry *entry;
@@ -1005,6 +1195,10 @@ gpm_power_get_battery_status (GpmPower			 *power,
 	return TRUE;
 }
 
+/**
+ * gpm_power_set_on_ac:
+ * @power: This power class instance
+ **/
 static gboolean
 gpm_power_set_on_ac (GpmPower *power,
 		     gboolean  on_ac,
@@ -1022,6 +1216,10 @@ gpm_power_set_on_ac (GpmPower *power,
 	return TRUE;
 }
 
+/**
+ * gpm_power_get_on_ac:
+ * @power: This power class instance
+ **/
 gboolean
 gpm_power_get_on_ac (GpmPower *power,
 		     gboolean *on_ac,
@@ -1036,6 +1234,9 @@ gpm_power_get_on_ac (GpmPower *power,
 	return TRUE;
 }
 
+/**
+ * gpm_power_set_property:
+ **/
 static void
 gpm_power_set_property (GObject	     *object,
 			guint	      prop_id,
@@ -1056,6 +1257,9 @@ gpm_power_set_property (GObject	     *object,
 	}
 }
 
+/**
+ * gpm_power_get_property:
+ **/
 static void
 gpm_power_get_property (GObject    *object,
 			guint       prop_id,
@@ -1076,13 +1280,14 @@ gpm_power_get_property (GObject    *object,
 	}
 }
 
+/**
+ * gpm_power_class_init:
+ **/
 static void
 gpm_power_class_init (GpmPowerClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize	   = gpm_power_finalize;
 	object_class->get_property = gpm_power_get_property;
 	object_class->set_property = gpm_power_set_property;
@@ -1139,6 +1344,15 @@ gpm_power_class_init (GpmPowerClass *klass)
 	g_type_class_add_private (klass, sizeof (GpmPowerPrivate));
 }
 
+/**
+ * hal_on_ac_changed_cb:
+ * @monitor: The HAL monitor class instance
+ * @on_ac: If we are on AC power
+ * @power: This power class instance
+ *
+ * As the state of the AC Adapter has changed, we need to refresh our device
+ * and kind caches as they have likely changed.
+ **/
 static void
 hal_on_ac_changed_cb (GpmHalMonitor *monitor,
 		      gboolean       on_ac,
@@ -1151,6 +1365,14 @@ hal_on_ac_changed_cb (GpmHalMonitor *monitor,
 	}
 }
 
+/**
+ * add_battery:
+ * @power: This power class instance
+ * @udi: The HAL UDI for this device
+ *
+ * Add a battery to a device cache, and then add the device cache to the
+ * correct kind cache depending on type.
+ **/
 static gboolean
 add_battery (GpmPower   *power,
 	     const char *udi)
@@ -1174,7 +1396,7 @@ add_battery (GpmPower   *power,
 	 * the design capacity. (#326740)
 	 */
 	if (entry->battery_kind == GPM_POWER_BATTERY_KIND_PRIMARY) {
-		if (status->capacity > 0 && status->capacity < 0.5f) {
+		if (status->capacity > 0 && status->capacity < 50) {
 			gpm_warning ("Your battery has a very low capacity, "
 				     "meaning that it may be old or broken. "
 				     "Battery life will be sub-optimal, "
@@ -1184,6 +1406,15 @@ add_battery (GpmPower   *power,
 	return TRUE;
 }
 
+/**
+ * remove_battery:
+ * @udi: The HAL UDI for this device
+ * @power: This power class instance
+ *
+ * Removes a battery from the device and kind caches from a UDI value.
+ *
+ * Return value: if a battery was removed.
+ **/
 static gboolean
 remove_battery (GpmPower   *power,
 		const char *udi)
@@ -1206,6 +1437,13 @@ remove_battery (GpmPower   *power,
 	return TRUE;
 }
 
+/**
+ * hal_battery_added_cb:
+ * @monitor: The HAL monitor class instance
+ * @udi: The HAL UDI for this device
+ * @power: This power class instance
+ * Called from HAL...
+ **/
 static void
 hal_battery_added_cb (GpmHalMonitor *monitor,
 		      const char    *udi,
@@ -1217,6 +1455,13 @@ hal_battery_added_cb (GpmHalMonitor *monitor,
 	battery_kind_cache_debug_print_all (power);
 }
 
+/**
+ * hal_battery_removed_cb:
+ * @monitor: The HAL monitor class instance
+ * @udi: The HAL UDI for this device
+ * @power: This power class instance
+ * Called from HAL...
+ **/
 static void
 hal_battery_removed_cb (GpmHalMonitor *monitor,
 			const char    *udi,
@@ -1233,6 +1478,14 @@ hal_battery_removed_cb (GpmHalMonitor *monitor,
 	g_signal_emit (power, signals [BATTERY_REMOVED], 0, udi);
 }
 
+/**
+ * hal_battery_property_modified_cb:
+ * @monitor: The HAL monitor class instance
+ * @udi: The HAL UDI for this device
+ * @key: The HAL key that is modified
+ * @power: This power class instance
+ * Called from HAL...
+ **/
 static void
 hal_battery_property_modified_cb (GpmHalMonitor *monitor,
 				  const char    *udi,
@@ -1259,7 +1512,7 @@ hal_battery_property_modified_cb (GpmHalMonitor *monitor,
 		return;
 	}
 
-	battery_device_cache_entry_update_key (device_entry, key);
+	battery_device_cache_entry_update_key (power, device_entry, key);
 
 	type_entry = battery_kind_cache_find (power, device_entry->battery_kind);
 
@@ -1273,6 +1526,13 @@ hal_battery_property_modified_cb (GpmHalMonitor *monitor,
 	battery_kind_cache_debug_print (type_entry);
 }
 
+/**
+ * hal_button_pressed_cb:
+ * @monitor: The HAL monitor class instance
+ * @type: the button type, e.g. "power"
+ * @power: This power class instance
+ * @state: The lid state, where TRUE is open or pressed
+ **/
 static void
 hal_button_pressed_cb (GpmHalMonitor *monitor,
 		       const char    *type,
@@ -1284,7 +1544,10 @@ hal_button_pressed_cb (GpmHalMonitor *monitor,
 	g_signal_emit (power, signals [BUTTON_PRESSED], 0, type, state);
 }
 
-/* FIXME: there must be a better way to do this */
+/**
+ * gpm_hash_remove_return:
+ * FIXME: there must be a better way to do this
+ **/
 static gboolean
 gpm_hash_remove_return (gpointer key,
 			gpointer value,
@@ -1293,6 +1556,10 @@ gpm_hash_remove_return (gpointer key,
 	return TRUE;
 }
 
+/**
+ * gpm_hash_new_kind_cache:
+ * @power: This power class instance
+ **/
 static void
 gpm_hash_new_kind_cache (GpmPower *power)
 {
@@ -1306,6 +1573,10 @@ gpm_hash_new_kind_cache (GpmPower *power)
 							 	 (GDestroyNotify)battery_kind_cache_entry_free);
 }
 
+/**
+ * gpm_hash_free_kind_cache:
+ * @power: This power class instance
+ **/
 static void
 gpm_hash_free_kind_cache (GpmPower *power)
 {
@@ -1319,6 +1590,10 @@ gpm_hash_free_kind_cache (GpmPower *power)
 	power->priv->battery_kind_cache = NULL;
 }
 
+/**
+ * gpm_hash_new_device_cache:
+ * @power: This power class instance
+ **/
 static void
 gpm_hash_new_device_cache (GpmPower *power)
 {
@@ -1332,6 +1607,10 @@ gpm_hash_new_device_cache (GpmPower *power)
 								   NULL);
 }
 
+/**
+ * gpm_hash_free_device_cache:
+ * @power: This power class instance
+ **/
 static void
 gpm_hash_free_device_cache (GpmPower *power)
 {
@@ -1345,18 +1624,58 @@ gpm_hash_free_device_cache (GpmPower *power)
 	power->priv->battery_device_cache = NULL;
 }
 
-static void
-hal_disconnected_cb (GpmHalMonitor *monitor,
-		     GpmPower      *power)
+/**
+ * gpm_power_update_all:
+ * @power: This power class instance
+ *
+ * We can call this anywhere to update all the device and kind caches
+ **/
+void
+gpm_power_update_all (GpmPower *power)
 {
-	/* We have to clear the caches, else the devices think they are
-	   initialised, and we segfault in various places. */
-	gpm_hash_free_kind_cache (power);
-	gpm_hash_new_kind_cache (power);
-	gpm_hash_free_device_cache (power);
-	gpm_hash_new_device_cache (power);
+	battery_kind_cache_update_all (power);
 }
 
+/**
+ * dbus_name_owner_changed_system_cb:
+ * @power: The power class instance
+ * @name: The DBUS name, e.g. hal.freedesktop.org
+ * @prev: The previous name, e.g. :0.13
+ * @new: The new name, e.g. :0.14
+ * @manager: This manager class instance
+ *
+ * The name-owner-changed system DBUS callback.
+ **/
+static void
+dbus_name_owner_changed_system_cb (GpmDbusSystemMonitor *dbus_monitor,
+				   const char	  *name,
+				   const char	  *prev,
+				   const char	  *new,
+				   GpmPower	  *power)
+{
+	if (strcmp (name, HAL_DBUS_SERVICE) == 0) {
+		if (strlen (prev) != 0 && strlen (new) == 0 ) {
+			hal_stop_monitor (power->priv->hal_monitor);
+			/* We have to clear the caches, else the devices think they are
+			   initialised, and we segfault in various places. */
+			gpm_hash_free_kind_cache (power);
+			gpm_hash_free_device_cache (power);
+		}
+		if (strlen (prev) == 0 && strlen (new) != 0 ) {
+			/* Re-create the caches */
+			gpm_hash_new_kind_cache (power);
+			gpm_hash_new_device_cache (power);
+			hal_start_monitor (power->priv->hal_monitor);
+			battery_kind_cache_update_all (power);
+		}
+	}
+}
+
+
+/**
+ * gpm_power_init:
+ * @power: This power class instance
+ **/
 static void
 gpm_power_init (GpmPower *power)
 {
@@ -1375,8 +1694,10 @@ gpm_power_init (GpmPower *power)
 			  G_CALLBACK (hal_battery_added_cb), power);
 	g_signal_connect (power->priv->hal_monitor, "battery-removed",
 			  G_CALLBACK (hal_battery_removed_cb), power);
-	g_signal_connect (power->priv->hal_monitor, "hal-disconnected",
-			  G_CALLBACK (hal_disconnected_cb), power);
+
+	power->priv->dbus_system = gpm_dbus_system_monitor_new ();
+	g_signal_connect (power->priv->dbus_system, "name-owner-changed",
+			  G_CALLBACK (dbus_name_owner_changed_system_cb), power);
 
 	power->priv->battery_kind_cache = NULL;
 	power->priv->battery_device_cache = NULL;
@@ -1386,8 +1707,17 @@ gpm_power_init (GpmPower *power)
 
 	on_ac = gpm_hal_monitor_get_on_ac (power->priv->hal_monitor);
 	gpm_power_set_on_ac (power, on_ac, NULL);
+
+	GConfClient *client = gconf_client_get_default ();
+	power->priv->exp_ave_factor = gconf_client_get_int (client,
+							    GPM_PREF_RATE_EXP_AVE_FACTOR,
+							    NULL);
+	g_object_unref (client);
 }
 
+/**
+ * gpm_power_finalize:
+ **/
 static void
 gpm_power_finalize (GObject *object)
 {
@@ -1400,16 +1730,19 @@ gpm_power_finalize (GObject *object)
 
 	g_return_if_fail (power->priv != NULL);
 
-	if (power->priv->hal_monitor != NULL) {
-		g_object_unref (power->priv->hal_monitor);
-	}
-
 	gpm_hash_free_kind_cache (power);
 	gpm_hash_free_device_cache (power);
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	g_object_unref (power->priv->hal_monitor);
+	g_object_unref (power->priv->dbus_system);
+
+	G_OBJECT_CLASS (gpm_power_parent_class)->finalize (object);
 }
 
+/**
+ * gpm_power_new:
+ * Return value: A new power class instance.
+ **/
 GpmPower *
 gpm_power_new (void)
 {
