@@ -48,40 +48,8 @@ gpm_stock_icons_init (void)
 	int           i;
 
 	static const GpmStockIcon items [] = {
-		/* GPM custom icons */
-		{ GPM_STOCK_AC_ADAPTER, TRUE },
-		{ GPM_STOCK_KEYBOARD_LOW, TRUE },
-		{ GPM_STOCK_MOUSE_LOW, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGED, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_000, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_020, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_040, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_060, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_080, TRUE },
-		{ GPM_STOCK_BATTERY_CHARGING_100, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_000, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_020, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_040, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_060, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_080, TRUE },
-		{ GPM_STOCK_BATTERY_DISCHARGING_100, TRUE },
-		{ GPM_STOCK_UPS_CHARGED, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_000, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_020, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_040, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_060, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_080, TRUE },
-		{ GPM_STOCK_UPS_CHARGING_100, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_000, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_020, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_040, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_060, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_080, TRUE },
-		{ GPM_STOCK_UPS_DISCHARGING_100, TRUE },
-		{ GPM_STOCK_POWER_INFORMATION, TRUE },
-		{ GPM_STOCK_SUSPEND_TO_DISK, TRUE },
-		{ GPM_STOCK_SUSPEND_TO_RAM, TRUE },
-
+		{ GPM_STOCK_SUSPEND_TO_DISK, FALSE },
+		{ GPM_STOCK_SUSPEND_TO_RAM, FALSE },
 	};
 
 	g_return_val_if_fail (factory == NULL, FALSE);
@@ -125,7 +93,6 @@ gpm_stock_icons_init (void)
 	return TRUE;
 }
 
-
 void
 gpm_stock_icons_shutdown (void)
 {
