@@ -41,7 +41,7 @@
 
 #include <libgnomeui/gnome-help.h> /* for gnome_help_display */
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
+#ifdef HAVE_LIBNOTIFY
 #include <libnotify/notify.h>
 #endif
 
@@ -71,9 +71,7 @@ struct GpmTrayIconPrivate
 	gboolean        can_suspend;
 	gboolean        can_hibernate;
 
-#if (LIBNOTIFY_VERSION_MINOR == 2)
-	NotifyHandle   *notify;
-#elif (LIBNOTIFY_VERSION_MINOR >= 3)
+#ifdef HAVE_LIBNOTIFY
 	NotifyNotification *notify;
 #endif
 };
@@ -81,6 +79,7 @@ struct GpmTrayIconPrivate
 enum {
 	SUSPEND,
 	HIBERNATE,
+	SHOW_INFO,
 	LAST_SIGNAL
 };
 
@@ -89,14 +88,9 @@ enum {
 	PROP_MODE
 };
 
-typedef enum {
-	LIBNOTIFY_URGENCY_CRITICAL = 1,
-	LIBNOTIFY_URGENCY_NORMAL   = 2,
-	LIBNOTIFY_URGENCY_LOW      = 3
-} LibNotifyEventType;
-
 static void gpm_tray_icon_suspend_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_hibernate_cb		(GtkAction *action, GpmTrayIcon *icon);
+static void gpm_tray_icon_show_info_cb	 	(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_preferences_cb	(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_help_cb		(GtkAction *action, GpmTrayIcon *icon);
 static void gpm_tray_icon_show_about_cb		(GtkAction *action, GpmTrayIcon *icon);
@@ -109,6 +103,8 @@ static GtkActionEntry gpm_tray_icon_action_entries [] =
 	  NULL, N_("Make the computer go to sleep"), G_CALLBACK (gpm_tray_icon_hibernate_cb) },
 	{ "TrayPreferences", GTK_STOCK_PREFERENCES, N_("_Preferences"),
 	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_preferences_cb) },
+	{ "TrayInfo", GTK_STOCK_DIALOG_INFO, N_("_Information"),
+	  NULL, NULL, G_CALLBACK (gpm_tray_icon_show_info_cb) },
 	{ "TrayHelp", GTK_STOCK_HELP, N_("_Help"), NULL,
 	  NULL, G_CALLBACK (gpm_tray_icon_show_help_cb) },
 	{ "TrayAbout", GTK_STOCK_ABOUT, N_("_About"), NULL,
@@ -116,11 +112,15 @@ static GtkActionEntry gpm_tray_icon_action_entries [] =
 };
 static guint gpm_tray_icon_n_action_entries = G_N_ELEMENTS (gpm_tray_icon_action_entries);
 
-static GObjectClass *parent_class = NULL;
 static guint         signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (GpmTrayIcon, gpm_tray_icon, EGG_TYPE_TRAY_ICON)
 
+/**
+ * gpm_tray_icon_enable_suspend:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should enable (i.e. show) the suspend icon
+ **/
 void
 gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 			      gboolean     enabled)
@@ -139,6 +139,11 @@ gpm_tray_icon_enable_suspend (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_enable_hibernate:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should enable (i.e. show) the hibernate icon
+ **/
 void
 gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 				gboolean     enabled)
@@ -158,6 +163,11 @@ gpm_tray_icon_enable_hibernate (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_set_tooltip:
+ * @icon: This TrayIcon class instance
+ * @tooltip: The tooltip text, e.g. "Batteries fully charged"
+ **/
 void
 gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
 			   const char  *tooltip)
@@ -170,6 +180,13 @@ gpm_tray_icon_set_tooltip (GpmTrayIcon *icon,
 			      tooltip, NULL);
 }
 
+/**
+ * gpm_tray_icon_set_image_from_stock:
+ * @icon: This TrayIcon class instance
+ * @stock_id: The icon name, e.g. GPM_STOCK_APP_ICON
+ *
+ * Loads a pixmap from disk, and sets as the tooltip icon
+ **/
 void
 gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 				    const char  *stock_id)
@@ -178,9 +195,17 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 	gpm_debug ("Setting icon to %s", stock_id);
 
 	if (stock_id) {
-		gtk_image_set_from_stock (GTK_IMAGE (icon->priv->image),
-					  stock_id,
-					  GTK_ICON_SIZE_LARGE_TOOLBAR);
+		GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+		GdkPixbuf *pixbuf;
+		pixbuf = gtk_icon_theme_load_icon (icon_theme, stock_id, 24, 0, NULL);
+		if (! pixbuf) {
+			gpm_critical_error ("Cannot find the file %s.\n"
+					    "possibly there is a packaging bug in g-p-m.",
+					    stock_id);
+		}
+		gtk_image_set_from_pixbuf (GTK_IMAGE (icon->priv->image), pixbuf );
+		g_object_unref (pixbuf);
+
 	} else {
 		/* FIXME: gtk_image_clear requires gtk 2.8, so until we
 		 * depend on more then 2.6 (required for FC4) we have to
@@ -192,6 +217,24 @@ gpm_tray_icon_set_image_from_stock (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_show_info_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
+static void
+gpm_tray_icon_show_info_cb (GtkAction   *action,
+			    GpmTrayIcon *icon)
+{
+	gpm_debug ("emitting show_info");
+	g_signal_emit (icon, signals [SHOW_INFO], 0);
+}
+
+/**
+ * gpm_tray_icon_hibernate_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_hibernate_cb (GtkAction   *action,
 			    GpmTrayIcon *icon)
@@ -200,6 +243,11 @@ gpm_tray_icon_hibernate_cb (GtkAction   *action,
 	g_signal_emit (icon, signals [HIBERNATE], 0);
 }
 
+/**
+ * gpm_tray_icon_suspend_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_suspend_cb (GtkAction   *action,
 			  GpmTrayIcon *icon)
@@ -208,6 +256,11 @@ gpm_tray_icon_suspend_cb (GtkAction   *action,
 	g_signal_emit (icon, signals [SUSPEND], 0);
 }
 
+/**
+ * gpm_tray_icon_show_preferences_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_show_preferences_cb (GtkAction   *action,
 				   GpmTrayIcon *icon)
@@ -219,6 +272,11 @@ gpm_tray_icon_show_preferences_cb (GtkAction   *action,
 	}
 }
 
+/**
+ * gpm_tray_icon_show_help_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_show_help_cb (GtkAction   *action,
 			    GpmTrayIcon *icon)
@@ -232,8 +290,13 @@ gpm_tray_icon_show_help_cb (GtkAction   *action,
 	}
 }
 
+/**
+ * gpm_tray_icon_show_about_cb:
+ * @action: A valid GtkAction
+ * @icon: This TrayIcon class instance
+ **/
 static void
-gpm_tray_icon_show_about_cb (GtkAction  *action,
+gpm_tray_icon_show_about_cb (GtkAction   *action,
 			     GpmTrayIcon *icon)
 {
 	const char *authors[] = {
@@ -273,12 +336,13 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 		translators = NULL;
 	}
 
-	pixbuf = gdk_pixbuf_new_from_file (GPM_DATA "gnome-power.png", NULL);
+	GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+	pixbuf = gtk_icon_theme_load_icon (icon_theme, GPM_STOCK_APP_ICON, 48, 0, NULL);
 
 	license_trans = g_strconcat (_(license[0]), "\n\n", _(license[1]), "\n\n",
 				     _(license[2]), "\n\n", _(license[3]), "\n",  NULL);
 
-	gtk_window_set_default_icon_from_file (GPM_DATA "gnome-power-manager.png", NULL);
+	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 
 	gtk_show_about_dialog (NULL,
 		               "name", GPM_NAME,
@@ -297,6 +361,11 @@ gpm_tray_icon_show_about_cb (GtkAction  *action,
 	g_object_unref (pixbuf);
 }
 
+/**
+ * tray_popup_position_menu:
+ *
+ * Popup the drop-down menu at the base of the icon
+ **/
 static void
 tray_popup_position_menu (GtkMenu  *menu,
 			  int      *x,
@@ -329,14 +398,29 @@ tray_popup_position_menu (GtkMenu  *menu,
 	*push_in = TRUE;
 }
 
+/**
+ * gpm_tray_icon_popup_cleared_cd:
+ * @widget: The popup Gtkwidget
+ * @icon: This TrayIcon class instance
+ *
+ * We have to re-enable the tooltip when the popup is removed
+ **/
 static void
-gpm_tray_icon_popup_cleared_cd (GtkWidget   *mo,
+gpm_tray_icon_popup_cleared_cd (GtkWidget   *widget,
 				GpmTrayIcon *icon)
 {
 	/* we enable the tooltip as the menu has gone */
 	gtk_tooltips_enable (icon->priv->tooltips);
 }
 
+/**
+ * gpm_tray_icon_button_press_cb:
+ * @widget: The tray icon widget
+ * @event: Valid event
+ * @icon: This TrayIcon class instance
+ *
+ * What do do when the button is left, right, middle clicked etc.
+ **/
 static gboolean
 gpm_tray_icon_button_press_cb (GtkWidget      *widget,
 			       GdkEventButton *event,
@@ -359,50 +443,35 @@ gpm_tray_icon_button_press_cb (GtkWidget      *widget,
 	return TRUE;
 }
 
+/**
+ * gpm_tray_icon_sync_actions:
+ * @icon: This TrayIcon class instance
+ *
+ * Syncs the private icon->priv->can_* variables with the icon states
+ **/
 static void
-gpm_tray_icon_set_property (GObject		  *object,
-			       guint		   prop_id,
-			       const GValue	  *value,
-			       GParamSpec	  *pspec)
-{
-	switch (prop_id) {
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-gpm_tray_icon_get_property (GObject		  *object,
-			       guint		   prop_id,
-			       GValue		  *value,
-			       GParamSpec	  *pspec)
-{
-	switch (prop_id) {
-	default:
-		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-		break;
-	}
-}
-
-static void
-gpm_tray_icon_sync_actions (GpmTrayIcon *tray)
+gpm_tray_icon_sync_actions (GpmTrayIcon *icon)
 {
 	GtkAction *action;
 
-	if (tray->priv->actiongroup != NULL) {
-		action = gtk_action_group_get_action (tray->priv->actiongroup,
+	if (icon->priv->actiongroup != NULL) {
+		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TraySuspend");
 
-		gtk_action_set_visible (GTK_ACTION (action), tray->priv->can_suspend);
+		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_suspend);
 
-		action = gtk_action_group_get_action (tray->priv->actiongroup,
+		action = gtk_action_group_get_action (icon->priv->actiongroup,
 						      "TrayHibernate");
 
-		gtk_action_set_visible (GTK_ACTION (action), tray->priv->can_hibernate);
+		gtk_action_set_visible (GTK_ACTION (action), icon->priv->can_hibernate);
 	}
 }
 
+/**
+ * gpm_tray_icon_constructor:
+ *
+ * Connects the UI to the tray icon instance
+ **/
 static GObject *
 gpm_tray_icon_constructor (GType                  type,
 			   guint                  n_construct_properties,
@@ -438,6 +507,7 @@ gpm_tray_icon_constructor (GType                  type,
 					   "    <menuitem action=\"TrayHibernate\" />"
 					   "    <separator />"
 					   "    <menuitem action=\"TrayPreferences\" />"
+					   "    <menuitem action=\"TrayInfo\" />"
 					   "    <separator />"
 					   "    <menuitem action=\"TrayHelp\" />"
 					   "    <menuitem action=\"TrayAbout\" />"
@@ -460,16 +530,15 @@ gpm_tray_icon_constructor (GType                  type,
 	return G_OBJECT (tray);
 }
 
+/**
+ * gpm_tray_icon_class_init:
+ **/
 static void
 gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 {
 	GObjectClass   *object_class = G_OBJECT_CLASS (klass);
 
-	parent_class = g_type_class_peek_parent (klass);
-
 	object_class->finalize	   = gpm_tray_icon_finalize;
-	object_class->get_property = gpm_tray_icon_get_property;
-	object_class->set_property = gpm_tray_icon_set_property;
 	object_class->constructor  = gpm_tray_icon_constructor;
 
 	g_type_class_add_private (klass, sizeof (GpmTrayIconPrivate));
@@ -494,8 +563,23 @@ gpm_tray_icon_class_init (GpmTrayIconClass *klass)
 		              g_cclosure_marshal_VOID__VOID,
 		              G_TYPE_NONE,
 		              0);
+	signals [SHOW_INFO] =
+		g_signal_new ("show-info",
+		              G_TYPE_FROM_CLASS (object_class),
+		              G_SIGNAL_RUN_LAST,
+		              G_STRUCT_OFFSET (GpmTrayIconClass, hibernate),
+		              NULL,
+		              NULL,
+		              g_cclosure_marshal_VOID__VOID,
+		              G_TYPE_NONE,
+		              0);
 }
 
+/**
+ * gpm_tray_icon_show:
+ * @icon: This TrayIcon class instance
+ * @enabled: If we should show the tray
+ **/
 void
 gpm_tray_icon_show (GpmTrayIcon *icon,
 		    gboolean     enabled)
@@ -510,6 +594,12 @@ gpm_tray_icon_show (GpmTrayIcon *icon,
 	}
 }
 
+/**
+ * gpm_tray_icon_init:
+ * @icon: This TrayIcon class instance
+ *
+ * Initialise the tray object, and set up libnotify
+ **/
 static void
 gpm_tray_icon_init (GpmTrayIcon *icon)
 {
@@ -536,16 +626,18 @@ gpm_tray_icon_init (GpmTrayIcon *icon)
 	gtk_container_add (GTK_CONTAINER (icon), icon->priv->ebox);
 	gpm_tray_icon_show (GPM_TRAY_ICON (icon), FALSE);
 
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
+#ifdef HAVE_LIBNOTIFY
 	ret = notify_init (GPM_NAME);
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-	ret = notify_glib_init (GPM_NAME, NULL);
 #endif
 	if (!ret) {
 		gpm_warning ("gpm_tray_icon_init failed");
 	}
 }
 
+/**
+ * gpm_tray_icon_finalize:
+ * @object: This TrayIcon class instance
+ **/
 static void
 gpm_tray_icon_finalize (GObject *object)
 {
@@ -560,21 +652,32 @@ gpm_tray_icon_finalize (GObject *object)
 
 	gtk_object_destroy (GTK_OBJECT (tray_icon->priv->tooltips));
 
-	G_OBJECT_CLASS (parent_class)->finalize (object);
+	G_OBJECT_CLASS (gpm_tray_icon_parent_class)->finalize (object);
 }
 
+/**
+ * gpm_tray_icon_new:
+ * Return value: A new TrayIcon object.
+ **/
 GpmTrayIcon *
 gpm_tray_icon_new (void)
 {
 	GpmTrayIcon *tray_icon;
-
 	tray_icon = g_object_new (GPM_TYPE_TRAY_ICON, NULL);
-
 	return GPM_TRAY_ICON (tray_icon);
 }
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
-static gboolean
+#ifdef HAVE_LIBNOTIFY
+/**
+ * get_widget_position:
+ * @widget: The icon widget
+ * @x: The returned X co-ordinate
+ * @y: The returned Y co-ordinate
+ *
+ * Gets the position where the libnotify arrow should "point"
+ * FIXME: Doesn't work on all orientations...
+ **/
+static void
 get_widget_position (GtkWidget *widget,
 		     int       *x,
 		     int       *y)
@@ -589,115 +692,71 @@ get_widget_position (GtkWidget *widget,
 	*y += widget->allocation.height;
 
 	gpm_debug ("widget position x=%i, y=%i", *x, *y);
-
-	return TRUE;
 }
-#endif
-
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
 
+/**
+ * notification_closed_cb:
+ * @notify: our libnotify instance
+ * @icon: This TrayIcon class instance
+ **/
 static void
 notification_closed_cb (NotifyNotification *notify,
-			GpmTrayIcon        *tray)
+			GpmTrayIcon        *icon)
 {
 	/* just invalidate the pointer */
 	gpm_debug ("caught notification closed signal");
-	tray->priv->notify = NULL;
+	icon->priv->notify = NULL;
 }
 
+/**
+ * libnotify_event:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a libnotify messagebox dialogue.
+ * Return value: success
+ **/
 static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
+libnotify_event (GpmTrayIcon    *icon,
+		 const char	*title,
+		 const char	*content,
+		 guint		 timeout,
+		 const char	*msgicon,
+		 GpmNotifyLevel	 urgency)
 {
 	int x;
 	int y;
 
-	if (tray->priv->notify != NULL) {
-		notify_notification_close (tray->priv->notify, NULL);
+	if (icon->priv->notify != NULL) {
+		notify_notification_close (icon->priv->notify, NULL);
 	}
 
-	tray->priv->notify = notify_notification_new (subject, content,
-						      GPM_STOCK_BATTERY_DISCHARGING_100,
+	icon->priv->notify = notify_notification_new (title,
+						      content,
+						      msgicon,
 						      NULL);
 
-	notify_notification_set_timeout (tray->priv->notify, timeout * 1000);
+	notify_notification_set_timeout (icon->priv->notify, timeout * 1000);
 
-	if (tray->priv->is_visible) {
-		get_widget_position (GTK_WIDGET (tray), &x, &y);
-		notify_notification_set_hint_int32 (tray->priv->notify, "x", x);
-		notify_notification_set_hint_int32 (tray->priv->notify, "y", y);
+	if (icon->priv->is_visible) {
+		get_widget_position (GTK_WIDGET (icon), &x, &y);
+		notify_notification_set_hint_int32 (icon->priv->notify, "x", x);
+		notify_notification_set_hint_int32 (icon->priv->notify, "y", y);
 	}
 
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
+	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
 		gpm_warning ("libnotify: %s : %s", GPM_NAME, content);
 	} else {
 		gpm_debug ("libnotify: %s : %s", GPM_NAME, content);
 	}
 
-	g_signal_connect (tray->priv->notify, "closed", G_CALLBACK (notification_closed_cb), tray);
+	g_signal_connect (icon->priv->notify, "closed", G_CALLBACK (notification_closed_cb), icon);
 
-	if (! notify_notification_show (tray->priv->notify, NULL)) {
-		gpm_warning ("failed to send notification (%s)", content);
-		return FALSE;
-	}
-
-	return TRUE;
-}
-
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-
-static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
-{
-	NotifyIcon  *icon = NULL;
-	NotifyHints *hints = NULL;
-	int          x;
-	int          y;
-
-	/* assertion checks */
-	g_assert (content);
-
-	hints = notify_hints_new ();
-	if (tray->priv->is_visible) {
-		get_widget_position (GTK_WIDGET (tray), &x, &y);
-		notify_hints_set_int (hints, "x", x);
-		notify_hints_set_int (hints, "y", y);
-	}
-	/* echo to terminal too */
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
-		gpm_warning ("libnotify: %s : %s", GPM_NAME, content);
-	} else {
-		gpm_debug ("libnotify: %s : %s", GPM_NAME, content);
-	}
-
-	/* use default g-p-m icon for now */
-	icon = notify_icon_new_from_uri (GPM_DATA "gnome-power.png");
-
-	if (tray->priv->notify != NULL) {
-		notify_close (tray->priv->notify);
-	}
-
-	tray->priv->notify = notify_send_notification (tray->priv->notify,
-						       NULL,
-						       urgency,
-						       subject,
-						       content,
-						       icon, /* icon */
-						       TRUE,
-						       timeout,
-						       hints, /* hints */
-						       NULL, /* no user data */
-						       0);   /* no actions */
-	notify_icon_destroy (icon);
-
-	if (! tray->priv->notify) {
+	if (! notify_notification_show (icon->priv->notify, NULL)) {
 		gpm_warning ("failed to send notification (%s)", content);
 		return FALSE;
 	}
@@ -707,13 +766,25 @@ libnotify_event (GpmTrayIcon             *tray,
 
 #else
 
+/**
+ * libnotify_event:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a gtk messagebox dialogue.
+ * Return value: success
+ **/
 static gboolean
-libnotify_event (GpmTrayIcon             *tray,
-		 guint                    timeout,	/* in seconds */
-		 const char              *subject,
-		 const char              *content,
-		 const LibNotifyEventType urgency)
-
+libnotify_event (GpmTrayIcon    *icon,
+		 const char	*title,
+		 const char	*content,
+		 guint		 timeout,
+		 const char	*msgicon,
+		 GpmNotifyLevel	 urgency)
 {
 	GtkWidget     *dialog;
 	GtkMessageType msg_type;
@@ -721,7 +792,7 @@ libnotify_event (GpmTrayIcon             *tray,
 	/* assertion checks */
 	g_assert (content);
 
-	if (urgency == LIBNOTIFY_URGENCY_CRITICAL) {
+	if (urgency == GPM_NOTIFY_URGENCY_CRITICAL) {
 		msg_type = GTK_MESSAGE_WARNING;
 	} else {
 		msg_type = GTK_MESSAGE_INFO;
@@ -734,7 +805,7 @@ libnotify_event (GpmTrayIcon             *tray,
 						     "<span size='larger'><b>%s</b></span>",
 						     GPM_NAME);
 
-	gtk_message_dialog_format_secondary_markup (GTK_MESSAGE_DIALOG (dialog), content);
+	gtk_message_dialog_format_content_markup (GTK_MESSAGE_DIALOG (dialog), content);
 
 	g_signal_connect_swapped (dialog,
 				  "response",
@@ -747,46 +818,58 @@ libnotify_event (GpmTrayIcon             *tray,
 }
 #endif
 
+/**
+ * gpm_tray_icon_notify:
+ * @icon: This icon class instance
+ * @title: The title, e.g. "Battery Low"
+ * @content: The contect, e.g. "17 minutes remaining"
+ * @timeout: The time we should remain on screen in seconds
+ * @msgicon: The icon to display, or NULL, e.g. GPM_STOCK_UPS_CHARGING_080
+ * @urgency: The urgency type, e.g. GPM_NOTIFY_URGENCY_CRITICAL
+ *
+ * Does a libnotify or gtk messagebox dialogue.
+ **/
 void
-gpm_tray_icon_notify (GpmTrayIcon *icon,
-		      guint        timeout,	/* in seconds */
-		      const char  *primary,
-		      GtkWidget   *msgicon,
-		      const char  *secondary)
+gpm_tray_icon_notify (GpmTrayIcon	*icon,
+		      const char	*title,
+		      const char	*content,
+		      guint		 timeout,
+		      const char	*msgicon,
+		      GpmNotifyLevel	 urgency)
 {
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
 
 	if (! icon->priv->show_notifications) {
-		gpm_debug ("ignoring notification: %s", primary);
+		gpm_debug ("ignoring notification: %s", title);
 		return;
 	}
 
-	gpm_debug ("doing notify: %s", primary);
+	gpm_debug ("doing notify: %s", title);
 
 	libnotify_event (icon,
+			 title,
+			 content,
 			 timeout,
-			 primary,
-			 secondary,
-			 LIBNOTIFY_URGENCY_CRITICAL);
+			 msgicon,
+			 urgency);
 }
 
+/**
+ * gpm_tray_icon_cancel_notify:
+ * @icon: This icon class instance
+ *
+ * Cancels the notification, i.e. removes it from the screen.
+ **/
 void
 gpm_tray_icon_cancel_notify (GpmTrayIcon *icon)
 {
 	GError *error;
-
 	g_return_if_fail (GPM_IS_TRAY_ICON (icon));
-
 	error = NULL;
 
-#if LIBNOTIFY_VERSION_MINOR >= 2
+#ifdef HAVE_LIBNOTIFY
 	if (icon->priv->notify != NULL) {
-#if (LIBNOTIFY_VERSION_MINOR >= 3)
 		notify_notification_close (icon->priv->notify, &error);
-#elif (LIBNOTIFY_VERSION_MINOR == 2)
-		notify_close (icon->priv->notify);
-		icon->priv->notify = NULL;
-#endif
 	}
 	if (error != NULL) {
 		g_error_free (error);
